/*
	MirandaRPC Plugin for Miranda-IM (http://www.miranda-im.org)
	Copyright 2004/2005 P. Boon

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
	MirandaRPC.c
	This file contain all RPC functions
	Interfaces in ../Interface/MirandaRPC.idl
*/

#include "MirandaRPCPlugin.h"
#include "../Interface/MirandaRPC.h"

/* [string][unique] */ unsigned char *MIM_getStatusDesc( 
/* [in] */ unsigned int in_statusCode) {

	char *szStatus;
	unsigned char *szRet;

	szStatus = (char *)CallService(MS_CLIST_GETSTATUSMODEDESCRIPTION, (WPARAM)in_statusCode, 0);
	if (szStatus == NULL) return NULL;
	szRet = (unsigned char *)midl_user_allocate(strlen(szStatus)+1);
	if (szRet == NULL) return NULL;
	strcpy(szRet, szStatus);

	return szRet;
}


unsigned int MIM_getMyStatusCode( 
    /* [string][unique][in] */ unsigned char *in_proto) {

	if (in_proto == NULL) 
		return CallService(MS_CLIST_GETSTATUSMODE, 0, 0);
	else if (CallService(MS_PROTO_ISPROTOCOLLOADED, 0, (LPARAM)in_proto))
		return CallProtoService(in_proto, PS_GETSTATUS, 0, 0);
	else /* failed to retrieve status */
		return ID_STATUS_UNKNOWN;
}


unsigned int MIM_getContactHandle( 
	/* [string][unique][in] */ unsigned char *in_contactDesc) {

	HANDLE hContact;

	hContact = getContactHandle(in_contactDesc);
	if (hContact == NULL) return 0;
	else return (unsigned int)hContact;
}


unsigned int MIM_getContactStatusCode( 
    /* [in] */ unsigned int in_hContact) {
	
	HANDLE hContact;
	char *szProto;

	hContact = (HANDLE)in_hContact;
	if (hContact == NULL) return ID_STATUS_UNKNOWN;
	szProto = (char *)CallService(MS_PROTO_GETCONTACTBASEPROTO, (WPARAM)hContact, 0);
	if (szProto == NULL) return ID_STATUS_UNKNOWN;

	return DBGetContactSettingWord(hContact, szProto, "Status", ID_STATUS_UNKNOWN);
}


/* [string][unique] */ unsigned char *MIM_getContactInfo( 
    /* [in] */ unsigned int in_hContact,
/* [in] */ unsigned int in_type) {

	unsigned char* ret;
	char *res;

	ret = NULL;
	res = getContactInfo((HANDLE)in_hContact, (BYTE)in_type);
	if (res != NULL) {
		ret = (unsigned char *)midl_user_allocate(strlen(res)+1);
		strcpy(ret, res);
		free(res);
	}
	
	return ret;
}


unsigned int MIM_getContactsInGroup( 
    /* [string][unique][in] */ unsigned char *in_group,
    /* [in] */ unsigned int in_sFlags,
    /* [out] */ long *out_size,
    /* [size_is][size_is][out] */ unsigned int **out_hContacts) {

	HANDLE hContact;
	unsigned int *hContacts;
	DBVARIANT dbv;
	unsigned int dbRes;
	long count;
	char *szProto;

	*out_size = count = 0;
	*out_hContacts = hContacts = NULL;
	hContact = (HANDLE)CallService(MS_DB_CONTACT_FINDFIRST,0,0);
	while (hContact != NULL) {
		dbRes = DBGetContactSetting(hContact, "CList", "Group", &dbv);
		if ( (in_group == NULL) || ( (!dbRes) && (!strcmp(in_group, dbv.pszVal))) ) {
			if (in_sFlags != 0)
				szProto = (char *)CallService(MS_PROTO_GETCONTACTBASEPROTO, (WPARAM)hContact, 0);
			if ( (in_sFlags == 0) || (statusModeToFlag(DBGetContactSettingWord(hContact, szProto, "Status", ID_STATUS_UNKNOWN))&in_sFlags) ) {
				if (count == 0) hContacts = (unsigned int *)midl_user_allocate(sizeof(unsigned int));
				else hContacts = (unsigned int *)realloc(hContacts, (count+1)*sizeof(unsigned int));
				if (hContacts == NULL) return 0;
				hContacts[count] = (unsigned int)hContact;
				count += 1;
			}
		}
		if (!dbRes) DBFreeVariant(&dbv);
		hContact = (HANDLE)CallService(MS_DB_CONTACT_FINDNEXT,(WPARAM)hContact, 0);
	}
	qsort(hContacts, count, sizeof(unsigned int), compareContacts);

	*out_size = count;
	*out_hContacts = hContacts;

	return count;
}


unsigned int MIM_getEventQueueInfo( 
    /* [in] */ unsigned int in_hContact,
    /* [in] */ unsigned int in_tFlags,
    /* [out] */ long *out_size,
    /* [size_is][size_is][out] */ MIM_EVENT_INFO **out_eventQueueInfo) {

	unsigned int eventCount, eventType, i;
	HANDLE hContact;
	CLISTEVENT *cle;
	MIM_EVENT_INFO *eqi;

	*out_size = eventCount = 0;
	*out_eventQueueInfo = eqi = NULL;

	if ((HANDLE)in_hContact == NULL) hContact = INVALID_HANDLE_VALUE;
	else hContact = (HANDLE)in_hContact;

	for(i=0;i<MAX_EVENTCOUNT;i++) {
		cle = (CLISTEVENT *)CallService(MS_CLIST_GETEVENT, (WPARAM)hContact, (LPARAM)i);
		if (cle == NULL) break;
		eventType = getEventType(cle->pszService);
		if ((eventType&in_tFlags) == 0) continue;
		if (eventCount == 0) eqi = (MIM_EVENT_INFO *)midl_user_allocate(sizeof(MIM_EVENT_INFO));
		else eqi = (MIM_EVENT_INFO *)realloc(eqi, (eventCount+1)*sizeof(MIM_EVENT_INFO));
		if (eqi == NULL) return 0;
		eqi[eventCount].hContact = (unsigned int)cle->hContact;
		eqi[eventCount].eventType = eventType;
		eventCount += 1;
	}
	*out_eventQueueInfo = eqi;
	*out_size = eventCount;

	return eventCount;
}

/* [string][unique] */ unsigned char *MIM_getDBStringValue( 
    /* [in] */ unsigned int in_hContact,
    /* [string][unique][in] */ unsigned char *in_module,
/* [string][unique][in] */ unsigned char *in_setting) {

	HANDLE hContact;
	DBVARIANT dbv;
	char* var;

	hContact = (HANDLE)in_hContact;
	if (DBGetContactSetting(hContact, in_module, in_setting, &dbv)) return NULL;
	switch (dbv.type) {
	case DBVT_BYTE:
		var = midl_user_allocate(12);
		if (var == NULL) return NULL;
		_ltoa(dbv.bVal,var,10);
		break;
	case DBVT_WORD:
		var = midl_user_allocate(12);
		if (var == NULL) return NULL;
		_ltoa(dbv.wVal,var,10);
		break;
	case DBVT_DWORD:
		var = midl_user_allocate(12);
		if (var == NULL) return NULL;
		_ltoa(dbv.dVal,var,10);
		break;
	case DBVT_ASCIIZ:
		var = midl_user_allocate(strlen(dbv.pszVal)+1);
		if (var == NULL) return NULL;
		strcpy(var, dbv.pszVal);
		break;
	}
	DBFreeVariant(&dbv);

	return var;
}

/* [string][unique] */ unsigned char *MIM_getVariablesFormattedString( 
    /* [in] */ unsigned int in_hContact,
/* [string][unique][in] */ unsigned char *in_string) {

	FORMATINFO fi;
	char *fString;
	unsigned char *ret;

	if (!ServiceExists(MS_VARS_FORMATSTRING))
		return NULL;
	ZeroMemory(&fi, sizeof(fi));
	fi.cbSize = sizeof(fi);
	fi.szFormat = in_string;
	fi.hContact = (HANDLE)in_hContact;
	fString = (char *)CallService(MS_VARS_FORMATSTRING, (WPARAM)&fi, 0);
	if (fString != NULL)
		ret = _strdup(fString);
	else
		ret = NULL;
	CallService(MS_VARS_FREEMEMORY, (WPARAM)fString, 0);

	return ret;
}


	
/******************************************************/
/*               Call from main thread                */
/******************************************************/

static void openDlg(WPARAM wParam, LPARAM lParam) {

	HANDLE hContact;
	char *msg;

	hContact = (HANDLE)wParam;
	msg = (char *)lParam;

	if (hContact != NULL) {
		if (ServiceExists("SRMsg/LaunchMessageWindow"))
			CallService("SRMsg/LaunchMessageWindow", (WPARAM)hContact, (LPARAM)msg);
		else
			CallService(MS_MSG_SENDMESSAGE, (WPARAM)hContact, (LPARAM)msg);
	}
	else
		CallService(MS_MSG_FORWARDMESSAGE, 0, (LPARAM)msg);

	if (msg != NULL)
		free(msg);
}

/******************************************************/
/*                     Control                        */
/******************************************************/

unsigned int MIM_sendMessage( 
    /* [in] */ unsigned int in_hContact,
    /* [string][unique][in] */ unsigned char *in_text) {

	char *msg;
	FUNCINFO *fInfo;

	msg = NULL;
	if (in_text != NULL)
		msg = _strdup(in_text);
	if (WaitForSingleObject(hPendingFuncMutex, INFINITE) == WAIT_OBJECT_0) {
		fInfo = (FUNCINFO *)malloc(sizeof(FUNCINFO));
		if (fInfo != NULL) {
			fInfo->func = openDlg;
			fInfo->wParam = (WPARAM)in_hContact;
			fInfo->lParam = (LPARAM)msg;
		}
		pFunc = fInfo;
		ReleaseMutex(hPendingFuncMutex);
	}
	SetEvent(hWakeMainThread);
	
	return 0;
}

unsigned int MIM_setStatus( 
    /* [string][unique][in] */ unsigned char *in_proto,
    /* [in] */ unsigned int in_statusCode) {

	if (in_proto == NULL)
		return CallService(MS_CLIST_SETSTATUSMODE, (WPARAM)in_statusCode, 0);
	else
		return CallProtoService(in_proto, PS_SETSTATUS, (WPARAM)in_statusCode, 0);
}

/******************************************************/
/*         MIDL allocate and free                     */
/******************************************************/
 
void __RPC_FAR * __RPC_USER midl_user_allocate(size_t len)
{
    return(malloc(len));
}
 
void __RPC_USER midl_user_free(void __RPC_FAR * ptr)
{
    free(ptr);
}

/******************************************************/
/*         Thread waiting for calls                   */
/******************************************************/

int initRPC() {

    RPC_STATUS status;
	DBVARIANT dbv;
	char *protoSeq, *endPoint;
	int maxCalls;
    
	protoSeq = endPoint = NULL;
	maxCalls = DBGetContactSettingWord(NULL, MODULENAME, SETTING_MAXCALLS, RPC_DEF_MAXCALLS);
	if (!DBGetContactSetting(NULL, MODULENAME, SETTING_PROTOSEQ, &dbv)) {
		protoSeq = _strdup(dbv.pszVal);
		DBFreeVariant(&dbv);
	}
	else protoSeq = _strdup(RPC_DEF_PROTOSEQ);
	if ( (protoSeq == NULL) || (RpcNetworkIsProtseqValid(protoSeq) != RPC_S_OK) )
		return -1;
	if (!DBGetContactSetting(NULL, MODULENAME, SETTING_ENDPOINT, &dbv)) {
		endPoint = _strdup(dbv.pszVal);
		DBFreeVariant(&dbv);
	}
	else {
		if (!strcmp(protoSeq, "ncalrpc"))
			endPoint = _strdup(RPC_DEF_ENDPOINT);
		else if (!strcmp(protoSeq, "ncacn_ip_tcp"))
			endPoint = _strdup(RPC_DEF_TCPENDPOINT);
	}
	if (endPoint == NULL) {
		free(protoSeq);	
		return -1;
	}
	status = RpcServerUseProtseqEp(protoSeq, maxCalls, endPoint, NULL);
	if (status != RPC_S_OK) return status;
	status = RpcServerRegisterIf(MirandaRPC_v1_0_s_ifspec, NULL, NULL);
	if (status != RPC_S_OK) return status;

	return 0;
}

int deInitRPC() {

	/* NOTE: this function doesn't remove the endpoint, this will cause an error when calling 
	RpcServerUseProtoSeqEp after calling deInitRPC */

	RPC_STATUS status;
	
	status = RpcMgmtStopServerListening(NULL);
	if (status != RPC_S_OK) return status;
	status = RpcServerUnregisterIf(MirandaRPC_v1_0_s_ifspec, NULL, FALSE);
	if (status != RPC_S_OK) return status;

	return 0;
}

DWORD WINAPI rpcThread(LPVOID arg) {
   
	int minCalls, maxCalls;

	minCalls = DBGetContactSettingWord(NULL, MODULENAME, SETTING_MINCALLS, RPC_DEF_MINCALLS);
	maxCalls = DBGetContactSettingWord(NULL, MODULENAME, SETTING_MAXCALLS, RPC_DEF_MAXCALLS);
	RpcServerListen(minCalls, maxCalls, FALSE);
	ExitThread(0);
		
	return 0;
}
