/*
	MirandaRPC Plugin for Miranda-IM (http://www.miranda-im.org)
	Copyright 2004/2005 P. Boon

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "MirandaRPCPlugin.h"

static CONTACTCE **cce = NULL;
static int contactCacheSize = 0;
extern HANDLE hContactCacheMutex;

static HANDLE *auc = NULL;
static int avatarUpdatingCacheSize = 0;
extern HANDLE hAvatarCacheMutex;

unsigned int statusModeToFlag(unsigned int status) {

	switch (status) {
	case ID_STATUS_OFFLINE:
		return SF_OFFLINE;
		break;
	case ID_STATUS_ONLINE:
		return SF_ONLINE;
		break;
	case ID_STATUS_INVISIBLE:
		return SF_INVISIBLE;
		break;
	case ID_STATUS_AWAY:
		return SF_AWAY;
		break;
	case ID_STATUS_NA:
		return SF_NA;
		break;
	case ID_STATUS_OCCUPIED:
		return SF_OCCUPIED;
		break;
	case ID_STATUS_DND:
		return SF_DND;
		break;
	case ID_STATUS_FREECHAT:
		return SF_FREECHAT;
		break;
	case ID_STATUS_OUTTOLUNCH:
		return SF_OUTTOLUNCH;
		break;
	case ID_STATUS_ONTHEPHONE:
		return SF_ONTHEPHONE;
		break;
	case ID_STATUS_UNKNOWN:
		return SF_UNKNOWN;
		break;
	}

	return 0;
}

int protoAck(WPARAM wParam, LPARAM lParam) {

	ACKDATA *ack=(ACKDATA*)lParam;
	int i;
		
	if (ack->type != ACKTYPE_AVATAR) return 0;
	if ( (ack->result != ACKRESULT_SUCCESS) && (ack->result != ACKRESULT_FAILED) ) return 0;
	if (WaitForSingleObject(hAvatarCacheMutex, INFINITE) == WAIT_OBJECT_0) {
		for (i=0;i<avatarUpdatingCacheSize;i++) {
			if (auc[i] == ack->hContact) { 
				/* invalidate the cache, the next time the protocol gets the request, it returns the info */
				/* ...I hope */
				if (avatarUpdatingCacheSize > 1)
					auc[i] = auc[avatarUpdatingCacheSize-1];
				avatarUpdatingCacheSize -= 1;
				if (avatarUpdatingCacheSize < 0) 
					avatarUpdatingCacheSize = 0; /* shouldn't happen */
				else
					auc = realloc(auc, avatarUpdatingCacheSize*sizeof(HANDLE));
				if (avatarUpdatingCacheSize == 0)
					auc = NULL;
				ReleaseMutex(hAvatarCacheMutex);
				return 0;
			}
		}
		ReleaseMutex(hAvatarCacheMutex);
	}

	return 0;
}

static char *getAvatarFilename(HANDLE hContact) {

	PROTO_AVATAR_INFORMATION ai = {0};
	int i;
	char *szProto;

	szProto = (char *)CallService(MS_PROTO_GETCONTACTBASEPROTO, (WPARAM)hContact, 0);
	if (szProto == NULL) return NULL;
	if (!(CallProtoService(szProto, PS_GETCAPS, (WPARAM)PFLAGNUM_4, 0)&PF4_AVATARS))
		return NULL;
	// prevent multiple outstanding avatar updates
	// maybe we already requested an updated (maybe the protocol should handle this)
	if (WaitForSingleObject(hAvatarCacheMutex, INFINITE) == WAIT_OBJECT_0) {
		for (i=0;i<avatarUpdatingCacheSize;i++) {
			if (auc[i] == hContact) {
				ReleaseMutex(hAvatarCacheMutex);
				return NULL;
			}
		}
		ReleaseMutex(hAvatarCacheMutex);
	}
	ai.cbSize = sizeof(ai);
	ai.hContact = hContact;
	switch (CallProtoService(szProto, PS_GETAVATARINFO, (WPARAM)GAIF_FORCE, (LPARAM)&ai)) {
	case GAIR_SUCCESS:
		return _strdup(ai.filename);
	case GAIR_NOAVATAR:
		return NULL;
	case GAIR_WAITFOR:
		// add to 'being updated' array
		if (WaitForSingleObject(hAvatarCacheMutex, INFINITE) == WAIT_OBJECT_0) {
			auc = realloc(auc, (avatarUpdatingCacheSize+1)*sizeof(HANDLE));
			auc[avatarUpdatingCacheSize] = hContact;
			avatarUpdatingCacheSize += 1;
			ReleaseMutex(hAvatarCacheMutex);
		}
		break;
	}

	return NULL;
}

char *getContactInfo(HANDLE hContact, BYTE type) {

	CONTACTINFO ci;
	char *res;

	if (type == CCNF_PROTOCOL) {
		char *szProto;

		szProto = (char *)CallService(MS_PROTO_GETCONTACTBASEPROTO, (WPARAM)hContact, 0);
		if (szProto == NULL) return NULL;
		res = malloc(128);
		if (res == NULL) return NULL;
		if (!CallProtoService(szProto, PS_GETNAME, (WPARAM)128, (LPARAM)res)) return res;
		else return NULL;
	}
	else if (type == CCNF_STATUS) {
		char *szStatus, *szProto;
		int status;

		szProto = (char *)CallService(MS_PROTO_GETCONTACTBASEPROTO, (WPARAM)hContact, 0);
		status = DBGetContactSettingWord(hContact, szProto, "Status", -1);
		if (status == -1) {
			res = malloc(32);
			_snprintf(res, 32, "%s", Translate("Unknown"));
		}
		else {
			szStatus = (char *)CallService(MS_CLIST_GETSTATUSMODEDESCRIPTION, (WPARAM)status, 0);
			res = _strdup(szStatus);
		}
		return res;
	}
	else if (type == CCNF_AVATAR) {
		return getAvatarFilename(hContact);
	}
	ZeroMemory(&ci,sizeof(CONTACTINFO));
	ci.cbSize = sizeof(CONTACTINFO);
	ci.hContact = (HANDLE)hContact;
	ci.dwFlag = type;
	if (CallService(MS_CONTACT_GETCONTACTINFO,(WPARAM)0,(LPARAM)&ci)) return NULL;
	res = NULL;
	switch (ci.type) {
		case CNFT_BYTE:
			res = malloc(32*sizeof(char));
			if (res == NULL) return NULL;
			_ltoa(ci.bVal, res, 10);
			break;
		case CNFT_WORD:
			res = malloc(32*sizeof(char));
			if (res == NULL) return NULL;
			_ltoa(ci.wVal, res, 10);
			break;
		case CNFT_DWORD:
			res = malloc(32*sizeof(char));
			if (res == NULL) return NULL;
			_ltoa(ci.dVal, res, 10);
			break;
		case CNFT_ASCIIZ:
			if (ci.pszVal != NULL) {
				res = malloc(strlen(ci.pszVal) + 1);
				if (res == NULL) return NULL;
				strcpy(res, ci.pszVal);
				if ( (type == CNF_DISPLAY) || (type == CNF_DISPLAYNC) ) {
					struct MM_INTERFACE mm;
				
					mm.cbSize=sizeof(struct MM_INTERFACE);
					CallService(MS_SYSTEM_GET_MMI,0,(LPARAM)&mm);
					mm.mmi_free(ci.pszVal);
				}
			}
			break;
	}

	return res;
}


HANDLE getContactHandle(char *contactDesc) {

	char *cur, *proto, *id, *contact;
	HANDLE hContact;
	int i;
	
	if (contactDesc == NULL) return NULL;
	contact = contactDesc;
	if (*contact != '[') return NULL;
	
	/* search the cache */
	if (WaitForSingleObject(hContactCacheMutex, INFINITE) == WAIT_OBJECT_0) {
		for (i=0;i<contactCacheSize;i++) {
			if (!strcmp(cce[i]->szContact, contactDesc)) {
				ReleaseMutex(hContactCacheMutex);
				return cce[i]->hContact;
			}
		}
		ReleaseMutex(hContactCacheMutex);
	}
	/* contact not found in cache */
	contact++;
	proto = cur = contact;
	while ( (*cur != ']') && (*cur != '\0') ) cur++;
	if ( (*cur == '\0') || (*(cur+1) == '\0') ) return NULL;
	proto = malloc(cur-contact+1);
	if (proto == NULL) return NULL;
	memset(proto, '\0', cur-contact+1);
	strncpy(proto, contact, cur-contact);
	contact = cur+1;
	hContact = (HANDLE)CallService(MS_DB_CONTACT_FINDFIRST,0,0);
	while (hContact != NULL) {
		if (CallService(MS_PROTO_ISPROTOONCONTACT, (WPARAM)hContact, (LPARAM)proto)) {
			id = getContactInfo(hContact, CNF_UNIQUEID);
			if (id != NULL) {
				if (!strcmp(id, contact)) {
					free(id);
					free(proto);
					/* store in cache */
					if (WaitForSingleObject(hContactCacheMutex, INFINITE) == WAIT_OBJECT_0) {
						cce = realloc(cce, (contactCacheSize+1)*sizeof(CONTACTCE*));
						if (cce != NULL) {
							cce[contactCacheSize] = malloc(sizeof(CONTACTCE));
							if (cce[contactCacheSize] != NULL) {
								cce[contactCacheSize]->hContact = hContact;
								cce[contactCacheSize]->szContact = _strdup(contactDesc);
								if (cce[contactCacheSize]->szContact != NULL) {
									contactCacheSize += 1;
								}
							}
						}
						ReleaseMutex(hContactCacheMutex);
					}
					/* done */
					return hContact;
				}
				free(id);
			}

		}
		hContact = (HANDLE)CallService(MS_DB_CONTACT_FINDNEXT,(WPARAM)hContact, 0);
	}
	free(proto);

	return NULL;
}


int contactSettingChanged(WPARAM wParam, LPARAM lParam) {

	int i;
	DBCONTACTWRITESETTING *dbw;
	char *uid, *szProto;
	
	uid = NULL;
	if (WaitForSingleObject(hContactCacheMutex, INFINITE) == WAIT_OBJECT_0) {
		for (i=0;i<contactCacheSize;i++) {
			if ((HANDLE)wParam != cce[i]->hContact) continue;
			dbw = (DBCONTACTWRITESETTING*)lParam;
			szProto = (char *)CallService(MS_PROTO_GETCONTACTBASEPROTO, wParam,0);
			if (szProto != NULL)
				uid = (char*)CallProtoService(szProto,PS_GETCAPS,PFLAG_UNIQUEIDSETTING,0);
			if ( ((int)uid!=CALLSERVICE_NOTFOUND&&uid) && (uid != NULL) && (!strcmp(dbw->szSetting, uid)) ) {
				/* invalidate the cache */
				free(cce[i]->szContact);
				cce[i]->hContact = cce[contactCacheSize-1]->hContact;
				cce[i]->szContact = cce[contactCacheSize-1]->szContact;
				contactCacheSize -= 1;
				if (contactCacheSize < 0)
					contactCacheSize = 0; /* shouldn't happen */
				else
					realloc(cce, (contactCacheSize)*sizeof(CONTACTCE*));
				if (contactCacheSize == 0)
					cce = NULL;
				ReleaseMutex(hContactCacheMutex);
				return 0;
			}
		}
		ReleaseMutex(hContactCacheMutex);
	}
	
	return 0;
}

unsigned short getEventType(char* pszService) {

	if (pszService == NULL) return EF_UNKNOWN;
	else if (!strcmp(pszService, "SRMsg/TypingMessage")) return EF_TYPING;
	else if (!strcmp(pszService, "SRMsg/ReadMessage")) return EF_MESSAGE;
	else if (!strcmp(pszService, "SRUrl/ReadUrl")) return EF_URL;
	else if (!strcmp(pszService, "UserOnline/Description")) return EF_USERONLINE;
	else if (!strcmp(pszService, "Auth/ShowRequest")) return EF_AUTH_SHOWREQ;
	else if (!strcmp(pszService, "Auth/ShowAdded")) return EF_AUTH_SHOWADDED;
	else if (!strcmp(pszService, "SRFile/RecvFile")) return EF_FILE;

	else return EF_UNKNOWN;
}

int compareContacts(const void *arg1, const void *arg2) {

	return CallService(MS_CLIST_CONTACTSCOMPARE, (WPARAM)*(HANDLE **)arg1, (LPARAM)*(HANDLE **)arg2);
}
