/*
	MirandaRPC Plugin for Miranda-IM (http://www.miranda-im.org)
	Copyright 2004/2005 P. Boon

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "MirandaRPCPlugin.h"
#include "../Interface/MirandaRPC.h"
#include "resource.h"
#include "../../debugplugin/m_debug.h"

HINSTANCE hInst;
PLUGINLINK *pluginLink;
PLUGININFO pluginInfo = {
	sizeof(PLUGININFO),
	"MirandaRPC",
	PLUGIN_MAKE_VERSION(0,1,0,0),
	"RPC functionality to control Miranda from an external application",
	"P Boon",
	"unregistered@users.sourceforge.net",
	" 2004/2005 P. Boon",
	"http://miranda-im.org/",
	0,		//not transient
	0		//doesn't replace a module
};

static HANDLE hControlListeningService;
static HANDLE hModulesLoadedHook;
static HANDLE hPreShutdownHook;
static HANDLE hContactSettingChanged;
static HANDLE hProtoAck;
static HANDLE hOptionsHook;
static HANDLE hThreadId;
static HANDLE hObjectSignaledService;

HANDLE hContactCacheMutex;
HANDLE hAvatarCacheMutex;

extern int initRPC();
extern int deInitRPC();
extern DWORD WINAPI rpcThread(LPVOID arg);
extern int optionsInit(WPARAM wParam, LPARAM lParam);
BOOL RPCThreadRunning = FALSE;

static int objectSignaled(WPARAM wParam, LPARAM lParam) {

	if (WaitForSingleObject(hPendingFuncMutex, INFINITE) == WAIT_OBJECT_0) {
		if (pFunc != NULL) {
			pFunc->func(pFunc->wParam, pFunc->lParam);
			free(pFunc);
		}
		pFunc = NULL;
		ReleaseMutex(hPendingFuncMutex);
	}
	ResetEvent(hWakeMainThread);

	return 0;
}

static int startThread() {

	hWakeMainThread = CreateEvent(NULL, TRUE, TRUE, NULL);
	ResetEvent(hWakeMainThread);
	pFunc = NULL;
	hPendingFuncMutex = CreateMutex(NULL, FALSE, NULL);
	CallService(MS_SYSTEM_WAITONHANDLE, (WPARAM)hWakeMainThread, (LPARAM)MS_RPC_OBJECTSIGNALED);
	hContactCacheMutex = CreateMutex(NULL, FALSE, NULL);
	hAvatarCacheMutex = CreateMutex(NULL, FALSE, NULL);
	hThreadId = CreateThread(NULL, 0, rpcThread, NULL, 0, NULL);

	return 0;
}

static int controlListening(WPARAM wParam, LPARAM lParam) {

	int res;
	char err[128];

	if  ( ((wParam == 0) && (!RPCThreadRunning)) || (lParam) ) {
		RPCThreadRunning = TRUE;
		res = initRPC();
		if (res) {
			sprintf(err, "Failed to initialize (error %d)", res);
			MessageBox(NULL, err, "MirandaRPC", MB_OK);
		}
		else
			startThread();
	}
	else if  ( ((wParam == 0) && (RPCThreadRunning)) || (!lParam) ) {
		RPCThreadRunning = FALSE;
		deInitRPC();
		WaitForSingleObject(hThreadId, INFINITE);
	}
	
	return 0;
}

static int preShutdown(WPARAM wParam, LPARAM lParam) {

	UnhookEvent(hOptionsHook);
	UnhookEvent(hPreShutdownHook);
	UnhookEvent(hContactSettingChanged);
	UnhookEvent(hProtoAck);
	CallService(MS_RPC_CONTROLLISTENING, 1, 0);
	DestroyServiceFunction(hControlListeningService);

	return 0;
}

static int modulesLoaded(WPARAM wparam,LPARAM lparam) {

//	CLISTMENUITEM cli;


	hObjectSignaledService = CreateServiceFunction(MS_RPC_OBJECTSIGNALED, objectSignaled);
	hControlListeningService = CreateServiceFunction(MS_RPC_CONTROLLISTENING, controlListening);
/*	ZeroMemory(&cli, sizeof(cli));
	cli.cbSize=sizeof(CLISTMENUITEM);
	cli.pszName = Translate("Start/Stop RPC");
	cli.hIcon = LoadImage(hInst, MAKEINTRESOURCE(IDI_RPCMENU), IMAGE_ICON, 16, 16, 0);
	if (cli.hIcon != NULL) {
		cli.pszService = MS_RPC_CONTROLLISTENING;
		cli.pszPopupName = NULL;
		CallService(MS_CLIST_ADDMAINMENUITEM, 0, (LPARAM)&cli);
	}*/
	UnhookEvent(hModulesLoadedHook);
	hContactSettingChanged = HookEvent(ME_DB_CONTACT_SETTINGCHANGED, contactSettingChanged);
	hProtoAck = HookEvent(ME_PROTO_ACK, protoAck);
		
	/* start thread */
	CallService(MS_RPC_CONTROLLISTENING, 1, 1);
	hPreShutdownHook = HookEvent(ME_SYSTEM_PRESHUTDOWN, preShutdown);
	hOptionsHook = HookEvent(ME_OPT_INITIALISE, optionsInit);
			
	return 0;
}

BOOL WINAPI DllMain(HINSTANCE hinstDLL,DWORD fdwReason,LPVOID lpvReserved) {

	hInst=hinstDLL;
	DisableThreadLibraryCalls(hInst);
	
	return TRUE;
}

__declspec(dllexport) PLUGININFO* MirandaPluginInfo(DWORD mirandaVersion) {
	
	return &pluginInfo;
}

int __declspec(dllexport) Load(PLUGINLINK *link) {


	pluginLink = link;
	hModulesLoadedHook = HookEvent(ME_SYSTEM_MODULESLOADED,  modulesLoaded);
			
	return 0;
}

int __declspec(dllexport) Unload(void) {

	return 0;
}