/*
	MirandaRPC Plugin for Miranda-IM (http://www.miranda-im.org)
	Copyright 2004/2005 P. Boon

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef _MIRANDARPC_H
#define _MIRANDARPC_H

#include "AggressiveOptimize.h"
#include <windows.h>
#include <stdio.h>

#include <newpluginapi.h>
#include <m_langpack.h>
#include <m_database.h>
#include <m_system.h>
#include <m_protosvc.h>
#include <m_options.h>
#include <m_contacts.h>
#include <m_protocols.h>
#include <m_clist.h>
#include <m_skin.h>
#include <m_userinfo.h>
#include <m_clui.h>
#include <m_message.h>
#include "../Interface/MirandaRPCCommon.h"
#include "../../variables/m_variables.h"

#define DEBUG_PLUGINNAME	"RPC"
//#include "../../debugplugin/m_debug.h"

#ifndef MS_MSG_FORWARDMESSAGE
#define MS_MSG_FORWARDMESSAGE  "SRMsg/ForwardMessage"
#endif

#define MODULENAME			"MirandaRPC"
#define SETTING_PROTOSEQ	"protoseq"
#define SETTING_MINCALLS	"mincalls"
#define SETTING_MAXCALLS	"maxcalls"
#define SETTING_ENDPOINT	"endpoint"

unsigned int statusModeToFlag(unsigned int status);
char *getContactInfo(HANDLE hContact, BYTE type);
int contactSettingChanged(WPARAM wParam, LPARAM lParam);
int protoAck(WPARAM wParam, LPARAM lParam);
HANDLE getContactHandle(char *contact);
unsigned short getEventType(char* pszService);
void sortContacts(unsigned int **hUnsortedContacts, int first, int n);
int compareContacts(const void *arg1, const void *arg2);

/*
	Thread Info

	Some functions (e.a. notifying an event hook) are not thread-safe. 
	Therefore, they must be called from the main thread. In order to do
	so, Miranda gets the hWakeMainThread HANDLE with the MS_WAITONHANDLE 
	service. This means, that whenever the hWakeMainThread event is 
	signaled, Miranda will call MS_RPC_OBJECTSIGNALED.

	MS_RPC_OBJECTSIGNALED is a service function defined as 
	objectSignaled. This function will call the function in the pFunc
	pointer. The memory for this structure must be dynamic allocated, 
	since it will be free'd afterwards.

	This pointer (pFunc) must be mutexed, so wait for the 
	hPendingFuncMutex before modifying it, also release this mutex when
	your done.

	EXAMPLE:
	
		FUNCINFO *fInfo;

		msg = NULL;
		if (in_text != NULL)
			msg = _strdup(in_text);
		// wait for the mutex
		if (WaitForSingleObject(hPendingFuncMutex, INFINITE) == WAIT_OBJECT_0) {
			// allocate the memory (will be free'd afterwards)
			fInfo = (FUNCINFO *)malloc(sizeof(FUNCINFO));
			if (fInfo != NULL) {
				// set the function and its parameters
				fInfo->func = openDlg;
				fInfo->wParam = (WPARAM)in_hContact;
				fInfo->lParam = (LPARAM)msg;
			}
			// set the pFunc variable
			pFunc = fInfo;
			// release the mutex
			ReleaseMutex(hPendingFuncMutex);
		}
		// signal the event
		SetEvent(hWakeMainThread);

	As you can see, the pFunc variable may be overwritten before its
	actually called. Pray it doesn't happen or write a queue.
*/
HANDLE hPendingFuncMutex;
HANDLE hWakeMainThread;
typedef struct {
	void (*func) (WPARAM wParam, LPARAM lParam);
	WPARAM wParam;
	LPARAM lParam;
} FUNCINFO;
FUNCINFO *pFunc;

typedef struct {
	char* szContact;
	HANDLE hContact;
} CONTACTCE; /* contact cache entry */

/* if wParam == 0; the RPC service will be switched
	else the RPC service will be set OFF if lParam == 0
	or ON otherwise */
#define MS_RPC_CONTROLLISTENING	"/RPC/ControlListening"

/* used to call a function from the main thread */
#define MS_RPC_OBJECTSIGNALED "/RPC/ObjectSignaled"


#endif
