/*
	Miranda-IM Plugin for Serious Samurize (http://www.samurize.com)
	Copyright 2004/2005 P. Boon

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* 
  NOTE:
  This plugin uses RPC in a very simplistic way, on each request it simply gets (polls) the data from Miranda.
*/

#include "SamMiranda.h"

static char *res = NULL;
static BOOL serverListening = FALSE;
static int serverCheckTime;
static HANDLE hCheckServerTimer = NULL;
static HANDLE hCheckServerThreadId = NULL;

/* thread checking if the server is still reachable; used when RPC are done over TCP */
static DWORD WINAPI checkServerThread(LPVOID arg) {

	LARGE_INTEGER liDueTime;

    liDueTime.QuadPart=-10000000*(serverCheckTime);
	hCheckServerTimer = CreateWaitableTimer(NULL, FALSE, NULL);
	if (!hCheckServerTimer)
		ExitThread(-1);
	if (!SetWaitableTimer(hCheckServerTimer, &liDueTime, serverCheckTime*1000, NULL, NULL, FALSE))
		ExitThread(-1);
	while (serverCheckTime > 0) {
		if (WaitForSingleObject(hCheckServerTimer, INFINITE) != WAIT_OBJECT_0) {
			ExitThread(-1);
		}
		serverListening = (RpcMgmtIsServerListening(hMirandaRPC_IfHandle) == RPC_S_OK)?TRUE:FALSE;
	}
	ExitThread(0);
}

BOOL APIENTRY DllMain(HANDLE hModule, DWORD ul_reason_for_call, LPVOID lpReserved) {

	RPC_STATUS status;
    char protoSeq[32],  address[128], endPoint[256], options[512], filename[MAX_PATH], err[128], *cur;
	static unsigned char *pszStringBinding = NULL;
	
	switch (ul_reason_for_call) {
		case DLL_PROCESS_ATTACH: 
			// get the ini file for this plugin and load settings
			GetModuleFileName((HMODULE)hModule, filename, sizeof(filename));
			for (cur=filename+strlen(filename);((*cur != '.') && (cur >= filename));cur--) { }
			if (cur != filename) strcpy(cur+1, "ini");
			GetPrivateProfileString("RPC", "ProtoSeq", RPC_DEF_PROTOSEQ, protoSeq, sizeof(protoSeq), filename);
			GetPrivateProfileString("RPC", "NetAddress", "", address, sizeof(address), filename);
			GetPrivateProfileString("RPC", "EndPoint", RPC_DEF_ENDPOINT, endPoint, sizeof(endPoint), filename);
			GetPrivateProfileString("RPC", "Options", "", options, sizeof(options), filename);
			serverCheckTime = GetPrivateProfileInt("RPC", "ServerCheck", 0, filename);
			status = RpcStringBindingCompose(NULL, (unsigned char *)protoSeq, (unsigned char *)address, (unsigned char *)endPoint, (unsigned char *)options, &pszStringBinding);
			if (!status) 
				status = RpcBindingFromStringBinding(pszStringBinding, &hMirandaRPC_IfHandle);
			if (status) {
				sprintf(err, "error composing string: %d", status);	
				MessageBox(NULL, err, "Miranda-IM plugin", MB_OK);
			}
			if (serverCheckTime != 0) {
				serverListening = (RpcMgmtIsServerListening(hMirandaRPC_IfHandle) == RPC_S_OK)?TRUE:FALSE;
				hCheckServerThreadId = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)checkServerThread, (LPVOID)&serverCheckTime, 0, NULL);
			}
			else 
				serverListening = TRUE;
			break;
		case DLL_PROCESS_DETACH: 
			if (serverCheckTime != 0) {
				serverCheckTime = 0;
				WaitForSingleObject(hCheckServerThreadId, INFINITE);
			}
			RpcStringFree(&pszStringBinding); 
			RpcBindingFree(&hMirandaRPC_IfHandle);
			if (res != NULL) 
				free(res);
 			break;
	}
    return TRUE;
}

SAMURIZE_PLUGIN_INT dllstartup(HWND hwnd, int dlltype) {

	return 0;
}

SAMURIZE_PLUGIN_INT dllshutdown(int id) {

	return 0;
}

SAMURIZE_PLUGIN_CHAR getinfo(int infonum) {
	
	switch (infonum) {
        case 0: return "Miranda-IM Plugin";
        case 1: return "UnregistereD";
        case 2: return "1.0";
        case 3: return "April 29 2004";
        case 4: return __DATE__;
        case 5: return "http://www.samurize.com/";
        case 6: return "unregistered@users.sourceforge.net";
	}
	return "";
}

SAMURIZE_PLUGIN_INT dlltype() {

	return 1;
}

SAMURIZE_PLUGIN_CHAR init() {	
     
	return "showContacts|showContactsCount|showMyStatusDesc|showMyStatusImage|showContactStatusDesc|showContactStatusImage|showEventQueueCount|showEventQueueImage|showDBValue|showContactInfo";
}


SAMURIZE_PLUGIN_CHAR getparam(char* func) {	

	if (!strcmp(func, "showContacts")) return "group|status|display|sort|";
	else if (!strcmp(func, "showContactsCount")) return "group|status|";
	else if (!strcmp(func, "showMyStatusDesc")) return "protocol|";
	else if (!strcmp(func, "showMyStatusImage")) return "protocol|";
	else if (!strcmp(func, "showContactStatusDesc")) return "contact|";
	else if (!strcmp(func, "showContactStatusImage")) return "contact|";
	else if (!strcmp(func, "showEventQueueCount")) return "contact|type|";
	else if (!strcmp(func, "showEventQueueImage")) return "contact|type|blink|";
	else if (!strcmp(func, "showDBValue")) return "contact|module|setting|";
	else if (!strcmp(func, "showContactInfo")) return "contact|display|";
	
	return "";
}

/******************************************************/
/*                 Samurize functions                 */
/******************************************************/

SAMURIZE_PLUGIN_CHAR showContacts(char *groupname, char *statusType, char *display, char *sort) {

	unsigned int *hContacts, sFlags, dType, sType, cCount, i;
	unsigned char *group;
	long count;
	char *err;
	CONTACTINFO *cList;
	
	err = "";
	if (serverListening == FALSE) 
		return err;
    if ( (groupname == NULL) || (strlen(groupname) == 0) ) 
		group = NULL;
	else 
		group = (unsigned char *)groupname;
	if ( (display == NULL) || (strlen(display) == 0) ) 
		dType = CNF_DISPLAY;
	else 
		dType = getContactInfoType(display);
	sFlags = getStatusFlags(statusType);
	sType = getSortType(sort);
	if ( (sType != 0) && (dType == sType) )
		sType = SORT_DISPLAY; // prevents getting the same data twice
	count = 0;
	hContacts = NULL;
	
	RpcTryExcept { // 1-2 RPCs (2 if sort != display)
		// get the number of contacts
		cCount = MIM_getContactsInGroup(group, sFlags, &count, &hContacts);
		if ((hContacts == NULL) || (cCount == 0)) 
			return err; // no contacts
		else {
			cList = (CONTACTINFO *)malloc(cCount*sizeof(CONTACTINFO));
			if (cList == NULL)
				return err;
			for (i=0;i<cCount;i++) { // create 'contact list' in memory
				cList[i].szDisplay = NULL;
				cList[i].hContact = hContacts[i];
				
				if (dType != CCNF_VARIABLES)
					cList[i].szDisplay = MIM_getContactInfo(cList[i].hContact, dType);
				else if (strlen(display) > 4)
					cList[i].szDisplay = MIM_getVariablesFormattedString(cList[i].hContact, (unsigned char *)display+4);
				cList[i].sortBy = sType;
				if (sType == SORT_STATUS)
					cList[i].status = MIM_getContactStatusCode(hContacts[i]);
				else if ( (sType != 0) && (sType != SORT_DISPLAY) )
					cList[i].szContactInfo = MIM_getContactInfo(hContacts[i], sType);
			}
			if (sType != 0) // sort the contacts
				qsort(cList, cCount, sizeof(CONTACTINFO), compareContacts);
			if (res != NULL) 
				*res = '\0';
			for (i=0;i<cCount;i++) { // create the output string
				if (cList[i].szDisplay != NULL) {
					if (res == NULL) {
						res = (char *)malloc(strlen((char *)cList[i].szDisplay) + 3);
						if (res == NULL) 
							return err;
						memset(res, '\0', strlen((char *)cList[i].szDisplay) + 3);
					}
					else
						res = (char *)realloc(res, strlen(res) + strlen((char *)cList[i].szDisplay) + 3);
					if (res == NULL) 
						return err;
					sprintf(res, "%s%s\r\n", res, cList[i].szDisplay);
					midl_user_free(cList[i].szDisplay);
					if ( (sType != 0) && (sType != SORT_STATUS) && (cList[i].szContactInfo != NULL) )
						midl_user_free(cList[i].szContactInfo);
				}
			}
			midl_user_free(hContacts);
		}
    }
    RpcExcept(1) {
		return err;
	}
	RpcEndExcept
 
	return res;
}


SAMURIZE_PLUGIN_CHAR showContactsCount(char *groupname, char *statusType) {

	unsigned int *hContacts, sFlags;
	unsigned char *group;
	long count, size;
	char *err;

	err = "0";
	if (serverListening == FALSE) 
		return err;
    if ( (groupname == NULL) || (strlen(groupname) == 0) ) 
		group = NULL;
	else 
		group = (unsigned char *)groupname;
	sFlags = getStatusFlags(statusType);
	size = count = 0;
	hContacts = NULL;
	
	RpcTryExcept { // 1 RPC
		count = MIM_getContactsInGroup(group, sFlags, &size, &hContacts);
		if (hContacts != NULL) 
			midl_user_free(hContacts);
    }
    RpcExcept(1) {
		return err;
	}
	RpcEndExcept

	res = (char *)realloc(res, 16);
	if (res == NULL) 
		return err;
	_ultoa(count, res, 10);
 
	return res;
}

SAMURIZE_PLUGIN_CHAR showMyStatusDesc(char *proto) {

	unsigned int statusCode;
	unsigned char *statusDesc;
	char *err;

	err = "Unknown";
	if (serverListening == FALSE) 
		return err;
	
	RpcTryExcept { // 2 RPCs
		if ( (proto == NULL) || (strlen(proto) == 0) )
			statusCode = MIM_getMyStatusCode(NULL);
		else
			statusCode = MIM_getMyStatusCode((unsigned char *)proto);
		statusDesc = MIM_getStatusDesc(statusCode);	// translates
		if (statusDesc != NULL) {
			res = (char *)realloc(res, strlen((char *)statusDesc) + 1);
			if (res == NULL) 
				return err;
			strcpy(res, (const char *)statusDesc);
			midl_user_free(statusDesc);
		}
		else 
			return err;
    }
    RpcExcept(1) {
		return err;
	}
	RpcEndExcept
 
	return res;
}

SAMURIZE_PLUGIN_CHAR showMyStatusImage(char *proto) {

	unsigned int status;
	char *statusDesc, *err;

	err = "";
	if (serverListening == FALSE) 
		return err;
	status = MAX_CONNECT_RETRIES+1;

	RpcTryExcept { // 1 RPC
		if ( (proto != NULL) && (strlen(proto) > 0) )
			status = MIM_getMyStatusCode((unsigned char *)proto);
		else
			status = MIM_getMyStatusCode(NULL);
	}
	RpcExcept(1) {
		return err;
	}
	RpcEndExcept

	statusDesc = getStatusDesc(status);
	if (proto == NULL) 
		proto = '\0';
	res = (char *)realloc(res, strlen(proto) + strlen(statusDesc) + 2);
	if (res == NULL) 
		return err;
	sprintf(res, "%s_%s", proto, statusDesc);
 
	return res;
}

SAMURIZE_PLUGIN_CHAR showContactStatusDesc(char *contact) {

	unsigned char *statusDesc;
	unsigned int hContact, statusCode;
	char *err;

	err = "Unknown";
	if (serverListening == FALSE) 
		return err;
	
	RpcTryExcept { // 3 RPCs
		hContact = MIM_getContactHandle((unsigned char *)contact);
		if ((HANDLE)hContact == NULL) 
			return err;
		statusCode = MIM_getContactStatusCode(hContact);
		statusDesc = MIM_getStatusDesc(statusCode);
		if (statusDesc == NULL) 
			return err;
		res = (char *)realloc(res, strlen((char *)statusDesc) + 1);
		if (res == NULL) 
			return err;
		strcpy(res, (const char *)statusDesc);
		midl_user_free(statusDesc);
	}
    RpcExcept(1) {
		return err;
	}
	RpcEndExcept
 
	return res;
}

SAMURIZE_PLUGIN_CHAR showContactStatusImage(char *contact) {

	unsigned int status, hContact;
	char *statusDesc, *cur, *proto, *err;
	
	err = "";
	status = MAX_CONNECT_RETRIES+1;
	if (serverListening == FALSE) 
		return err;

	RpcTryExcept { // 2 RPCs
		hContact = MIM_getContactHandle((unsigned char *)contact);
		if ((HANDLE)hContact != NULL)
			status = MIM_getContactStatusCode(hContact);
	}
	RpcExcept(1) {
		return err;
	}
	RpcEndExcept
	
	if ( (contact != NULL) && (*contact == '[') ) {
		contact++;
		proto = cur = contact;
		while ( (*cur != ']') && (*cur != '\0') ) cur++;
		if ( (*cur != '\0') && (*(cur+1) != '\0') ) {
			proto = (char *)malloc(cur-contact+1);
			if (proto != NULL) {
				memset(proto, '\0', cur-contact+1);
				strncpy(proto, contact, cur-contact);
			}
		}
	}	
	statusDesc = getStatusDesc(status);
	if (proto == NULL) 
		proto = '\0';
	res = (char *)realloc(res, strlen(proto) + strlen(statusDesc) + 2);
	if (res == NULL) 
		return err;
	sprintf(res, "%s_%s", proto, statusDesc);
	free(proto);

	return res;
}

SAMURIZE_PLUGIN_CHAR showEventQueueCount(char *contact, char *type) {

	unsigned int hContact, eventCount, tFlags;
	long size;
	MIM_EVENT_INFO *eventQueueInfo;
	char *err;

	err = "0";
	if (serverListening == FALSE) 
		return err;
	tFlags = getEventFlags(type);
	eventQueueInfo = NULL;

	RpcTryExcept { // 2 RPCs
		hContact = MIM_getContactHandle((unsigned char *)contact);
		eventCount = MIM_getEventQueueInfo(hContact, tFlags, &size, &eventQueueInfo);
		if (eventQueueInfo != NULL)
			midl_user_free(eventQueueInfo);
	}
	RpcExcept(1) {
		return err;
	}
	RpcEndExcept

	res = (char *)realloc(res, 16);
	if (res == NULL) 
		return err;
	_ultoa(eventCount, res, 10);

	return res;
}

SAMURIZE_PLUGIN_CHAR showEventQueueImage(char *contact, char *type, char *blink) {

	unsigned int hContact, eventCount, tFlags, eventType;
	long size;
	char *typeDesc, *err;
	MIM_EVENT_INFO *eventQueueInfo;
	static BOOL blinkHide = FALSE; // yeah, this goes wrong when using this function in multiple meters
	
	err = "";
	if (serverListening == FALSE)
		return err;
	blinkHide = blinkHide?FALSE:TRUE;
	if ( ((blinkHide) && ( (blink != NULL) && (*blink == 'y'))) )
		return err;
	tFlags = getEventFlags(type);
	eventQueueInfo = NULL;
	eventType = 0;

	RpcTryExcept { // 1 RPC
		hContact = MIM_getContactHandle((unsigned char *)contact);
		eventCount = MIM_getEventQueueInfo(hContact, tFlags, &size, &eventQueueInfo);
		if ( (eventCount > 0) && (eventQueueInfo != NULL) ) {
			eventType = eventQueueInfo[0].eventType;
			midl_user_free(eventQueueInfo);
		}
	}
	RpcExcept(1) {
		return err;
	}
	RpcEndExcept
	
	typeDesc = getTypeDesc(eventType);
	res = (char *)realloc(res, strlen(typeDesc) + 1);
	if (res == NULL) 
		return err;
	sprintf(res, "%s", typeDesc);

	return res;
}

SAMURIZE_PLUGIN_CHAR showDBValue(char *contact, char *module, char *setting) {

	unsigned int hContact;
	unsigned char *val;
	char *err;

	err = "";
	if (serverListening == FALSE) 
		return err;

	RpcTryExcept { // 1 RPC
		if ( (contact == NULL) || (strlen(contact) == 0) ) 
			hContact = (unsigned int)NULL;
		else
			hContact = MIM_getContactHandle((unsigned char *)contact);
		val = MIM_getDBStringValue(hContact, (unsigned char *)module, (unsigned char *)setting);
		if (val != NULL) {
			res = (char *)realloc(res, strlen((char *)val) + 1);
			if (res == NULL) 
				return err;
			strcpy(res, (char *)val);
			midl_user_free(val);
		}
	}
	RpcExcept(1) {
		return err;
	}
	RpcEndExcept

	return res;
}

SAMURIZE_PLUGIN_CHAR showContactInfo(char *contact, char *display) {


	unsigned int hContact, dType;
	unsigned char *cInfo;
	MIM_EVENT_INFO *eventQueueInfo;
	long size;
	char *err;

	err = "";
	if (serverListening == FALSE) 
		return err;
	if ( (display == NULL) || (strlen(display) == 0) ) 
		dType = CNF_DISPLAY;
	else 
		dType = getContactInfoType(display);
	eventQueueInfo = NULL;
	cInfo = NULL;

	RpcTryExcept { // 2 RPCs
		if ( (contact == NULL) || (strlen(contact) == 0) && (dType != CCNF_VARIABLES) ) {
			if ( (MIM_getEventQueueInfo(0, 0xFFFFFFFF, &size, &eventQueueInfo) > 0) &&
				(eventQueueInfo != NULL) ) {
				hContact = eventQueueInfo[0].hContact;
				midl_user_free(eventQueueInfo);
			}
			else 
				return err;
		}
		else
			hContact = MIM_getContactHandle((unsigned char *)contact);
		if (dType != CCNF_VARIABLES)
			cInfo = MIM_getContactInfo(hContact, dType);
		else if (strlen(display) > 4)
			cInfo = MIM_getVariablesFormattedString(hContact, (unsigned char *)display+4);
		if (cInfo == NULL)
			return err;
		else {
			res = (char *)realloc(res, strlen((char *)cInfo)+1);
			if (res == NULL) 
				return err;
			strcpy(res, (char *)cInfo);
			midl_user_free(cInfo);
		}
	}
	RpcExcept(1) {
		return err;
	}
	RpcEndExcept

	return res;
}

/******************************************************/
/*         MIDL allocate and free                     */
/******************************************************/
 
void __RPC_FAR * __RPC_USER midl_user_allocate(size_t len)
{
    return(malloc(len));
}
 
void __RPC_USER midl_user_free(void __RPC_FAR * ptr)
{
    free(ptr);
}

