/*
	MirandaRPC Plugin for Miranda-IM (http://www.miranda-im.org)
	Copyright 2004/2005 P. Boon

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "resource.h"
#include "MirandaRPCPlugin.h"
#include "../Interface/MirandaRPC.h"

extern HANDLE hInst;
static char protoSeqs[][20] = {"ncalrpc", "ncacn_ip_tcp"};

static BOOL CALLBACK OptDlgProc(HWND hwndDlg,UINT msg,WPARAM wParam,LPARAM lParam) {

	static char* protoSeq = NULL;

	switch(msg) {
	case WM_INITDIALOG: {
		DBVARIANT dbv;
		int i, item;

		TranslateDialogDefault(hwndDlg);
		for (i=0;i<sizeof(protoSeqs)/sizeof(protoSeqs[0]);i++) {
			if (RpcNetworkIsProtseqValid(protoSeqs[i]) != RPC_S_OK) continue;
  			item = SendDlgItemMessage(hwndDlg, IDC_PROTOSEQ, CB_ADDSTRING, 0, (LPARAM)protoSeqs[i]);
			SendDlgItemMessage(hwndDlg, IDC_PROTOSEQ, CB_SETITEMDATA, item, i);
			if (!DBGetContactSetting(NULL, MODULENAME, SETTING_PROTOSEQ, &dbv)) {
				if (!strcmp(protoSeqs[i], dbv.pszVal)) {
					SendDlgItemMessage(hwndDlg, IDC_PROTOSEQ, CB_SETCURSEL, item, 0);
					protoSeq = protoSeqs[i];
				}
				DBFreeVariant(&dbv);
			}
		}
		if (protoSeq == NULL) {
			SendDlgItemMessage(hwndDlg, IDC_PROTOSEQ, CB_SETCURSEL, 0, 0);
			item = SendDlgItemMessage(hwndDlg, IDC_PROTOSEQ, CB_GETITEMDATA, SendDlgItemMessage(hwndDlg, IDC_PROTOSEQ, CB_GETCURSEL, 0, 0), 0);
			if (item == CB_ERR) MessageBox(hwndDlg, "Your OS does not support the correct RPC protocol sequences", MODULENAME, MB_OK);
			else protoSeq = protoSeqs[item];
		}
		SetDlgItemInt(hwndDlg, IDC_MINCALLS, DBGetContactSettingWord(NULL, MODULENAME, SETTING_MINCALLS, RPC_DEF_MINCALLS), FALSE);
		SetDlgItemInt(hwndDlg, IDC_MAXCALLS, DBGetContactSettingWord(NULL, MODULENAME, SETTING_MAXCALLS, RPC_DEF_MAXCALLS), FALSE);
		if (!DBGetContactSetting(NULL, MODULENAME, SETTING_ENDPOINT, &dbv)) {
			SetDlgItemText(hwndDlg, IDC_ENDPOINT, dbv.pszVal);
			DBFreeVariant(&dbv);
		}
		else if (protoSeq != NULL) {
			if (!strcmp(protoSeq, "ncalrpc"))
				SetDlgItemText(hwndDlg, IDC_ENDPOINT, RPC_DEF_ENDPOINT);
			else if (!strcmp(protoSeq, "ncacn_ip_tcp"))
				SetDlgItemText(hwndDlg, IDC_ENDPOINT, RPC_DEF_TCPENDPOINT);
		}
						}
		break;
	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDC_PROTOSEQ:
			protoSeq = protoSeqs[SendDlgItemMessage(hwndDlg, IDC_PROTOSEQ, CB_GETITEMDATA, SendDlgItemMessage(hwndDlg, IDC_PROTOSEQ, CB_GETCURSEL, 0, 0), 0)];
			if (HIWORD(wParam) == CBN_SELCHANGE) {
				if (!strcmp(protoSeq, "ncalrpc"))
					SetDlgItemText(hwndDlg, IDC_ENDPOINT, RPC_DEF_ENDPOINT);
				else if (!strcmp(protoSeq, "ncacn_ip_tcp"))
					SetDlgItemText(hwndDlg, IDC_ENDPOINT, RPC_DEF_TCPENDPOINT);
				SendMessage(GetParent(hwndDlg), PSM_CHANGED, 0, 0);
			}
			break;
		case IDC_MINCALLS:
		case IDC_MAXCALLS:
		case IDC_ENDPOINT:
			if ( (HIWORD(wParam) == EN_CHANGE) && ((HWND)lParam == GetFocus()) ) SendMessage(GetParent(hwndDlg), PSM_CHANGED, 0, 0);
			break;
		}
		break;
	case WM_NOTIFY:
		if (((LPNMHDR)lParam)->code == PSN_APPLY) {
			char *endPoint;
			int len;

			DBWriteContactSettingWord(NULL, MODULENAME, SETTING_MINCALLS, (WORD)GetDlgItemInt(hwndDlg, IDC_MINCALLS, NULL, FALSE));
			DBWriteContactSettingWord(NULL, MODULENAME, SETTING_MAXCALLS, (WORD)GetDlgItemInt(hwndDlg, IDC_MAXCALLS, NULL, FALSE));
			if (protoSeq != NULL)
				DBWriteContactSettingString(NULL, MODULENAME, SETTING_PROTOSEQ, protoSeq);
			DBDeleteContactSetting(NULL, MODULENAME, SETTING_ENDPOINT);
			len = SendDlgItemMessage(hwndDlg, IDC_ENDPOINT, WM_GETTEXTLENGTH, 0, 0);
			if (len > 0) {
				endPoint = malloc(len+1);
				memset(endPoint, '\0', len+1);
				if (endPoint != NULL) {
					GetDlgItemText(hwndDlg, IDC_ENDPOINT, endPoint, len+1);
					DBWriteContactSettingString(NULL, MODULENAME, SETTING_ENDPOINT, endPoint);
				}
			}
		}
		break;
	}

	return 0;
}


int optionsInit(WPARAM wparam,LPARAM lparam) {
	
	OPTIONSDIALOGPAGE odp;

	ZeroMemory(&odp,sizeof(odp));
	odp.cbSize=sizeof(odp);
	odp.position=150000000;
	odp.pszGroup=Translate("Plugins");
	odp.groupPosition=910000000;
	odp.hInstance=hInst;
	odp.pszTemplate=MAKEINTRESOURCE(IDD_OPTIONS);
	odp.pszTitle=Translate("MirandaRPC");
	odp.pfnDlgProc=OptDlgProc;
	odp.flags=ODPF_BOLDGROUPS;
	CallService(MS_OPT_ADDPAGE,wparam,(LPARAM)&odp);

	return 0;
}